/*
  Project:  4x3 matrix membrane keypad / security code
  Function: This sketch takes input from a keypad for secret code.  If the code is
           entered correctly it turns on a green LED to simulate access granted and
           prints access granted in Serial Monitor. Otherwise the red LED is lit and
           access denied is printed in Serial Monitor.
*/
//**********************************************************************************
#include <Keypad.h>//include library code
#include <EEPROM.h>//include library code
//**********************************************************************************
const int greenledPin = 10; //green LED attached to digital pin 10
const int redledPin = 11;   //red LED attached to digital pin 11
char password[8];
char pass[8], pass1[8];
int i = 0;
char customKey = 0;
const byte ROWS = 4; //four rows
const byte COLS = 3; //three columns
char hexaKeys[ROWS][COLS] = {//define the symbols on the buttons of the keypads
{'1','2','3'},
{'4','5','6'},
{'7','8','9'},
{'*','0','#'}
};
byte rowPins[ROWS] = {9, 8, 7, 6}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {5, 4, 3};    //connect to the column pinouts of the keypad
Keypad customKeypad = Keypad( makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);//initialize an instance of class NewKeypad
//************************************************************************************************
void setup()
{
  pinMode(greenledPin, OUTPUT);         //sets greenledPin as OUTPUT
  pinMode(redledPin, OUTPUT);           //sets redledPin as OUTPUT
  Serial.begin(9600);                   //initialise the serial communication at 9600 bps
  Serial.println("********************");
  Serial.println("*****Electronic*****");
  Serial.println("*****Keypad Lock****");
  Serial.println("********************");
  delay(2000);
  Serial.println("*****Enter code*****");
  for (int j = 0; j < 8; j++) //password set to 8 symbols: default is 12345678
    EEPROM.write(j, j + 49);  //if you want to have a different password lengh - change number 8 everywere in void loop() to your number 
  for (int j = 0; j < 8; j++)
    pass[j] = EEPROM.read(j);
}

void loop()
{
  customKey = customKeypad.getKey();
  if (customKey == '#')
    change();
  if (customKey)
  {
    password[i++] = customKey;
    Serial.print(customKey);
  }
  if (i == 8)
  {
    delay(200);
    for (int j = 0; j < 8; j++)
      pass[j] = EEPROM.read(j);
    if (!(strncmp(password, pass, 8)))
    {
      digitalWrite(greenledPin, HIGH);
      Serial.println("");
      Serial.println("********************");
      Serial.println("** ACCESS GRANTED **");
      Serial.println("**   WELCOME!!    **");
      Serial.println("********************");
      delay(2000);
      Serial.println("Use # to change code");
      delay(2000);
      Serial.println("********************");
      Serial.println("*****Enter code*****");
      i = 0;
      digitalWrite(greenledPin, LOW);
    }

    else
    {
      digitalWrite(redledPin, HIGH);
      Serial.println("");
      Serial.println("********************");
      Serial.println("** ACCESS DENIED! **");
      Serial.println("**  INVALID CODE  **");
      Serial.println("********************");
      Serial.println("Use # to change code");
      delay(2000);
      Serial.println("********************");
      Serial.println("*****Enter code*****");
      i = 0;
      digitalWrite(redledPin, LOW);
    }
  }
}
void change()
{
  int j = 0;
  Serial.println("*Enter current code*");
  while (j < 8)
  {
    char key = customKeypad.getKey();
    if (key)
    {
      pass1[j++] = key;
      Serial.print(key);
    }
    key = 0;
  }
  delay(500);

  if ((strncmp(pass1, pass, 8)))
  {
    Serial.println("");
    Serial.println("********************");
    Serial.println("** ACCESS DENIED! **");
    Serial.println("**  INVALID CODE  **");
    Serial.println("********************");
    delay(1000);
  }
  else
  {
    j = 0;
    Serial.println("");
    Serial.println("***Enter new code***");
    while (j < 8)
    {
      char key = customKeypad.getKey();
      if (key)
      {
        pass[j] = key;
        Serial.print(key);
        EEPROM.write(j, key);
        j++;
      }
    }
    Serial.println("");
    Serial.println("**Your code changed*");
    Serial.println("********************");
    delay(1000);
  }
  Serial.println("*****Enter code*****");
  customKey = 0;
}
